import 'package:flutter/material.dart';
import 'package:superellipse_shape/superellipse_shape.dart';

import '../../mcg_extension.dart';

class OverAlertModel {
  AlertType type;
  String? title;
  String message;
  bool autoClose;
  int duration;
  Widget? child;

  OverAlertModel({
    this.title,
    required this.message,
    this.autoClose = true,
    this.type = AlertType.info,
    this.duration = 3000,
  });

  OverAlertModel.child({
    this.child,
    this.duration = 300000,
  })  : type = AlertType.info,
        autoClose = false,
        message = '';
}

class OverAlertWidget extends StatelessWidget {
  final OverAlertModel model;
  OverAlertWidget(this.model);

  static const Color _sucColor = Color(0xff65BA6B);
  static const Color _sucBorder = Color(0xffC4E3C9);
  static const Color _infoColor = Color(0xff2D6ADD);
  static const Color _infoBorder = Color(0xffB2CCEE);
  static const Color _warnColor = Color(0xffE19936);
  static const Color _warnBorder = Color(0xffF4DEB4);
  static const Color _dangerColor = Color(0xffDA5A39);
  static const Color _dangerBorder = Color(0xffEDC7BD);
  @override
  Widget build(BuildContext context) {
    if (model.child != null) {
      return Padding(
        padding: const EdgeInsets.symmetric(vertical: 8.0, horizontal: 16),
        child: model.child!,
      );
    }

    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8.0, horizontal: 16),
      child: Material(
        color: (model.type == AlertType.successful
                ? _sucBorder
                : model.type == AlertType.info
                    ? _infoBorder
                    : model.type == AlertType.warning
                        ? _warnBorder
                        : _dangerBorder)
            .withAlpha(240),
        shape: OutlineInputBorder(
            borderRadius: BorderRadius.circular(16),
            borderSide: BorderSide(
              width: 1.5,
              color: model.type == AlertType.successful
                  ? _sucBorder
                  : model.type == AlertType.info
                      ? _infoBorder
                      : model.type == AlertType.warning
                          ? _warnBorder
                          : _dangerBorder,
            )),
        child: Row(
          children: [
            Container(
              width: 48,
              height: 48,
              alignment: Alignment.center,
              decoration: ShapeDecoration(
                  shape: SuperellipseShape(borderRadius: BorderRadius.circular(44)),
                  color: model.type == AlertType.successful
                      ? _sucColor
                      : model.type == AlertType.info
                          ? _infoColor
                          : model.type == AlertType.warning
                              ? _warnColor
                              : _dangerColor),
              child: Icon(
                model.type == AlertType.successful
                    ? Icons.check_circle_rounded
                    : model.type == AlertType.info
                        ? Icons.info_rounded
                        : model.type == AlertType.warning
                            ? Icons.warning_rounded
                            : Icons.dangerous_rounded,
                color: Colors.white,
                size: 36,
              ),
            ),
            16.widthBox,
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  if (model.title != null) model.title.text.bold.color(Colors.black).fontSize(20).make(),
                  model.message.text.color(Colors.black).fontSize(20).make(),
                ],
              ),
            ),
            Icons.clear_rounded.icon
                .onPressed(() {
                  OverAlert.remove(model);
                })
                .color(const Color(0xff343C45))
                .make()
          ],
        ).p8,
      ),
    );
  }
}
