part of '../bottomsheetpanel.dart';

class _FormBottomSheetPanel extends StatefulWidget {
  final String? title;
  final String? subTitle;
  final List<BottomSheetFormGroup> groups;
  final BottomSheetPanelHeight bottomSheetPanelHeight;
  final bool barrierDismissible;
  final bool dense;
  final bool multipleItemBackgrounds;
  final String? okButtonText;

  _FormBottomSheetPanel({this.title, this.subTitle, required this.groups, this.bottomSheetPanelHeight = BottomSheetPanelHeight.L, this.barrierDismissible = true, this.dense = true, this.multipleItemBackgrounds = true, this.okButtonText});

  @override
  State<_FormBottomSheetPanel> createState() => _FormBottomSheetPanelState();
}

class _FormBottomSheetPanelState<T> extends State<_FormBottomSheetPanel> {
  @override
  void initState() {
    widget.groups.forEach((element) {
      if (element.type == BottomSheetGroupLayoutType.multipleSelectList) {
        _data[element.groupKey] = [];
      }
    });
    super.initState();
  }

  final Map _data = {};

  @override
  Widget build(BuildContext context) {
    Widget _current = Padding(
      padding: const EdgeInsets.only(left: 16, right: 16, bottom: 16),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          if (widget.title != null || widget.subTitle != null) PanelHeaderWidget(title: widget.title, subTitle: widget.subTitle).pb12,
          Flexible(
            child: SingleChildScrollView(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  ...widget.groups.map((group) {
                    return Padding(
                      padding: EdgeInsets.symmetric(vertical: 12),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          if (group.groupName != null) group.groupName.text.fontSize(18).bold.make().pb2,
                          ClipRRect(
                            borderRadius: BorderRadius.circular(_kBorderRadius),
                            child: ListView.separated(
                              physics: BouncingScrollPhysics(),
                              padding: EdgeInsets.all(0),
                              shrinkWrap: true,
                              itemBuilder: (context, index) {
                                final _item = group.items[index];
                                late bool _itemIsSelected;
                                if (group.type == BottomSheetGroupLayoutType.simpleList || group.type == BottomSheetGroupLayoutType.row) {
                                  _itemIsSelected = _data[group.groupKey] == _item.value;
                                } else if (group.type == BottomSheetGroupLayoutType.multipleSelectList) {
                                  _itemIsSelected = (_data[group.groupKey] as List).contains(_item.value);
                                } else {
                                  throw ('E:aj');
                                }
                                final _borderRadius = group.items.length == 1
                                    ? BorderRadius.circular(_kBorderRadius)
                                    : index == 0
                                        ? BorderRadius.only(topLeft: Radius.circular(_kBorderRadius), topRight: Radius.circular(_kBorderRadius))
                                        : index == group.items.length - 1
                                            ? BorderRadius.only(bottomLeft: Radius.circular(_kBorderRadius), bottomRight: Radius.circular(_kBorderRadius))
                                            : BorderRadius.circular(0);
                                return MaterialButton(
                                  elevation: 0,
                                  highlightElevation: 0,
                                  hoverElevation: 0,
                                  shape: RoundedRectangleBorder(borderRadius: _borderRadius),
                                  color: widget.multipleItemBackgrounds == false
                                      ? Fav.design.sheet.card
                                      : index.isEven
                                          ? Fav.design.sheet.card
                                          : Fav.design.sheet.card.withAlpha(100),
                                  materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
                                  visualDensity: VisualDensity.compact,
                                  onPressed: () {
                                    if (_item.value != null) {
                                      setState(() {
                                        if (group.type == BottomSheetGroupLayoutType.simpleList || group.type == BottomSheetGroupLayoutType.row) {
                                          _data[group.groupKey] = _item.value;
                                        } else if (group.type == BottomSheetGroupLayoutType.multipleSelectList) {
                                          if (!(_data[group.groupKey] as List).contains(_item.value)) {
                                            (_data[group.groupKey] as List).add(_item.value!);
                                          } else {
                                            (_data[group.groupKey] as List).remove(_item.value);
                                          }
                                        } else {
                                          throw ('E:aj');
                                        }
                                      });
                                    }
                                  },
                                  // borderRadius: _borderRadius,
                                  child: Padding(
                                    padding: Inset(widget.dense ? 0 : 16),
                                    child: Row(
                                      crossAxisAlignment: CrossAxisAlignment.center,
                                      children: [
                                        if (_item.icon != null) _item.icon!.icon.color(_item.itemColor ?? Fav.design.sheet.headerText).padding(0).make(),
                                        if (_item.icon != null) (widget.dense ? 4 : 16).widthBox,
                                        Expanded(child: _item.name.text.color(_item.itemColor ?? Fav.design.sheet.contentText).setBold(_item.bold).align(_item.icon != null || _item.suffix != null ? TextAlign.start : TextAlign.center).make()),
                                        if (_item.suffix != null) (widget.dense ? 4 : 16).widthBox,
                                        if (_item.suffix != null) _item.suffix!,
                                        (widget.dense ? 4 : 16).widthBox,
                                        (_itemIsSelected ? Icons.check_box_rounded : Icons.check_box_outline_blank).icon.color(_itemIsSelected ? Fav.design.sheet.headerText : Fav.design.primaryText.withAlpha(80)).padding(0).make(),
                                      ],
                                    ),
                                  ),
                                );
                              },
                              separatorBuilder: (context, index) {
                                return Container(
                                  width: double.infinity,
                                  height: 0.5,
                                  color: Fav.design.primaryText.withAlpha(10),
                                );
                              },
                              itemCount: group.items.length,
                            ),
                          ),
                        ],
                      ),
                    );
                  }).toList()
                ],
              ),
            ),
          ),
          8.heightBox,
          ElevatedButton(
            style: ElevatedButton.styleFrom(
              shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
              tapTargetSize: MaterialTapTargetSize.shrinkWrap,
              backgroundColor: Fav.design.sheet.headerText,
            ),
            onPressed: () {
              OverBottomSheet.selectBottomSheetItem(_data);
            },
            child: Center(child: (widget.okButtonText ?? 'ok'.translate.toUpperCase()).text.fontSize(18).color(Colors.white).make().py12),
          ),
        ],
      ),
    );

    return _current;
  }
}
