part of '../bottomsheetpanel.dart';

class _ListBottomSheetPanel extends StatefulWidget {
  final String? title;
  final String? subTitle;
  final List<BottomSheetItem> items;
  final BottomSheetPanelHeight bottomSheetPanelHeight;
  final bool barrierDismissible;
  final bool dense;
  final bool multipleItemBackgrounds;

  _ListBottomSheetPanel({this.title, this.subTitle, required this.items, this.bottomSheetPanelHeight = BottomSheetPanelHeight.L, this.barrierDismissible = true, this.dense = true, this.multipleItemBackgrounds = true});

  @override
  State<_ListBottomSheetPanel> createState() => _ListBottomSheetPanelState();
}

class _ListBottomSheetPanelState<T> extends State<_ListBottomSheetPanel> {
  String? filterText;
  late bool _searchBarEnable;

  @override
  void initState() {
    _searchBarEnable = widget.items.length > 20;
    super.initState();
  }

  void onFilterTextChange(String text) {
    setState(() {
      filterText = text.toSearchCase();
    });
  }

  @override
  Widget build(BuildContext context) {
    final _filteredList = filterText.safeLength < 1 ? widget.items : widget.items.where((element) => element.name.toSearchCase().contains(filterText!)).toList();

    Widget _current = Padding(
      padding: const EdgeInsets.only(left: 16, right: 16, bottom: 16),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          if (widget.title != null || widget.subTitle != null) PanelHeaderWidget(title: widget.title, subTitle: widget.subTitle).pb12,
          if (_searchBarEnable)
            SearchTextField(
              onChanged: onFilterTextChange,
              style: TextStyle(color: Fav.design.primaryText),
              //! Overlay widget gerekir acarsan
              enableInteractiveSelection: false,
            ).pb12,
          Flexible(
            child: ClipRRect(
              borderRadius: BorderRadius.circular(_kBorderRadius),
              child: ListView.separated(
                physics: BouncingScrollPhysics(),
                padding: EdgeInsets.all(0),
                shrinkWrap: true,
                itemBuilder: (context, index) {
                  final _item = _filteredList[index];

                  final _borderRadius = _filteredList.length == 1
                      ? BorderRadius.circular(_kBorderRadius)
                      : index == 0
                          ? BorderRadius.only(topLeft: Radius.circular(_kBorderRadius), topRight: Radius.circular(_kBorderRadius))
                          : index == _filteredList.length - 1
                              ? BorderRadius.only(bottomLeft: Radius.circular(_kBorderRadius), bottomRight: Radius.circular(_kBorderRadius))
                              : BorderRadius.circular(0);
                  return MaterialButton(
                    elevation: 0,
                    highlightElevation: 0,
                    hoverElevation: 0,
                    shape: RoundedRectangleBorder(borderRadius: _borderRadius),
                    color: widget.multipleItemBackgrounds == false
                        ? Fav.design.sheet.card.withAlpha(50)
                        : index.isEven
                            ? Fav.design.sheet.card.withAlpha(50)
                            : Fav.design.sheet.card.withAlpha(100),
                    materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
                    visualDensity: VisualDensity.compact,
                    onPressed: () {
                      OverBottomSheet.selectBottomSheetItem(_item.value);
                    },
                    // borderRadius: _borderRadius,
                    child: Padding(
                      padding: Inset(widget.dense ? 0 : 16),
                      child: Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          if (_item.icon != null) _item.icon!.icon.color(_item.itemColor ?? Fav.design.sheet.headerText).padding(0).make(),
                          if (_item.icon != null) (widget.dense ? 4 : 16).widthBox,
                          Expanded(child: _item.name.text.color(_item.itemColor ?? Fav.design.sheet.contentText).setBold(_item.bold).align(_item.icon != null || _item.suffix != null ? TextAlign.start : TextAlign.center).make()),
                          if (_item.suffix != null) (widget.dense ? 4 : 16).widthBox,
                          if (_item.suffix != null) _item.suffix!,
                        ],
                      ),
                    ),
                  );
                },
                separatorBuilder: (context, index) {
                  return Container(
                    width: double.infinity,
                    height: 0.5,
                    color: Fav.design.sheet.card.withAlpha(250),
                  );
                },
                itemCount: _filteredList.length,
              ),
            ),
          ),
        ],
      ),
    );

    if (_searchBarEnable) {
      _current = GestureDetector(
          onTap: () {
            FocusScope.of(context).requestFocus(FocusNode());
          },
          child: _current);
    }
    // _current = Scaffold(
    //   body: GestureDetector(
    //       onTap: () {
    //         FocusScope.of(context).requestFocus(FocusNode());
    //       },
    //       child: _current),
    // );

    return _current;
  }
}
