part of '../bottomsheetpanel.dart';

class _SimpleGroupBottomSheetPanel extends StatelessWidget {
  final String? title;
  final String? subTitle;
  final List<BottomSheetGroup> groups;
  final BottomSheetPanelHeight bottomSheetPanelHeight;
  final bool barrierDismissible;
  final bool dense;

  _SimpleGroupBottomSheetPanel({
    this.title,
    this.subTitle,
    required this.groups,
    this.bottomSheetPanelHeight = BottomSheetPanelHeight.L,
    this.barrierDismissible = true,
    this.dense = true,
  });

  @override
  Widget build(BuildContext context) {
    Widget _current = Padding(
      padding: const EdgeInsets.only(left: 16, right: 16, bottom: 16),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          if (title != null || subTitle != null) PanelHeaderWidget(title: title, subTitle: subTitle).pb12,
          Flexible(
            child: SingleChildScrollView(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  ...groups.map((group) {
                    return Padding(
                      padding: EdgeInsets.symmetric(vertical: 12),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          if (group.groupName != null) group.groupName.text.fontSize(18).bold.make().pb2,
                          ClipRRect(
                            borderRadius: BorderRadius.circular(_kBorderRadius),
                            child: ListView.separated(
                              physics: BouncingScrollPhysics(),
                              padding: EdgeInsets.all(0),
                              shrinkWrap: true,
                              itemBuilder: (context, index) {
                                final _item = group.items[index];

                                final _borderRadius = group.items.length == 1
                                    ? BorderRadius.circular(_kBorderRadius)
                                    : index == 0
                                        ? BorderRadius.only(topLeft: Radius.circular(_kBorderRadius), topRight: Radius.circular(_kBorderRadius))
                                        : index == group.items.length - 1
                                            ? BorderRadius.only(bottomLeft: Radius.circular(_kBorderRadius), bottomRight: Radius.circular(_kBorderRadius))
                                            : BorderRadius.circular(0);
                                return MaterialButton(
                                  elevation: 0,
                                  highlightElevation: 0,
                                  hoverElevation: 0,
                                  shape: RoundedRectangleBorder(borderRadius: _borderRadius),
                                  color: Fav.design.sheet.card,
                                  materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
                                  visualDensity: VisualDensity.compact,
                                  onPressed: () {
                                    OverBottomSheet.selectBottomSheetItem(_item.value);
                                  },
                                  // borderRadius: _borderRadius,
                                  child: Padding(
                                    padding: Inset(dense ? 0 : 16),
                                    child: Row(
                                      crossAxisAlignment: CrossAxisAlignment.center,
                                      children: [
                                        if (_item.icon != null) _item.icon!.icon.color(_item.itemColor ?? Fav.design.sheet.headerText).padding(0).make(),
                                        if (_item.icon != null) (dense ? 4 : 16).widthBox,
                                        Expanded(child: _item.name.text.color(_item.itemColor ?? Fav.design.sheet.contentText).setBold(_item.bold).align(_item.icon != null || _item.suffix != null ? TextAlign.start : TextAlign.center).make()),
                                        if (_item.suffix != null) (dense ? 4 : 16).widthBox,
                                        if (_item.suffix != null) _item.suffix!,
                                        (dense ? 4 : 16).widthBox,
                                      ],
                                    ),
                                  ),
                                );
                              },
                              separatorBuilder: (context, index) {
                                return Container(
                                  width: double.infinity,
                                  height: 0.5,
                                  color: Fav.design.primaryText.withAlpha(10),
                                );
                              },
                              itemCount: group.items.length,
                            ),
                          ),
                        ],
                      ),
                    );
                  }).toList()
                ],
              ),
            ),
          ),
          8.heightBox,
        ],
      ),
    );

    return _current;
  }
}
