import 'package:flutter/material.dart';

import '../../mcg_extension.dart';
import '../../widgets/editablesearchtextfield.dart';
import 'other/header_widget.dart';

part 'dialog_panels/child.dart';
//* Istedigin kadar type olusturabilirsin.

part 'dialog_panels/form.dart';
part 'dialog_panels/list.dart';
part 'dialog_panels/multipleselectlist.dart';
part 'dialog_panels/simple.dart';
part 'dialog_panels/default.dart';

const _kBorderRadius = 12.0;

class DialogPanel extends StatelessWidget implements IPanel {
  final Widget? child;
  final String? title;
  final String? subTitle;
  final List<BottomSheetItem>? items;
  final List<BottomSheetFormGroup>? groups;
  final int _type;
  final BottomSheetPanelHeight bottomSheetPanelHeight;
  final bool? dense;
  final bool? multipleItemBackgrounds;
  final String? okButtonText;
  final bool useNavigator;

  ///Kaydirma acik olacak fakat cizgi istemiyorsan kullanabilirsin
  final bool removePill;
  final String? cancelText;
  final Color? primaryColor;

  /// Eger geri birsey  dondurecek bir tus koyacaksan overbottomsheetde geri dondurmek icin tus var
  DialogPanel.child({required this.child, this.bottomSheetPanelHeight = BottomSheetPanelHeight.L, this.barrierDismissible = true, this.removePill = false})
      : title = null,
        subTitle = null,
        items = null,
        dense = null,
        groups = null,
        multipleItemBackgrounds = null,
        okButtonText = null,
        useNavigator = true,
        primaryColor = null,
        cancelText = null,
        _type = 0;

  DialogPanel.defaultPanel({this.title, this.subTitle, this.okButtonText, this.primaryColor, this.cancelText, this.bottomSheetPanelHeight = BottomSheetPanelHeight.L, this.barrierDismissible = true, this.removePill = false})
      : child = null,
        dense = null,
        groups = null,
        items = null,
        multipleItemBackgrounds = null,
        useNavigator = false,
        _type = 5;

  DialogPanel.simpleList({this.title, this.subTitle, required this.items, this.bottomSheetPanelHeight = BottomSheetPanelHeight.L, this.barrierDismissible = true, this.removePill = false})
      : child = null,
        dense = null,
        groups = null,
        multipleItemBackgrounds = null,
        okButtonText = null,
        useNavigator = false,
        primaryColor = null,
        cancelText = null,
        _type = 1;

  DialogPanel.list({
    this.title,
    this.subTitle,
    required this.items,
    this.bottomSheetPanelHeight = BottomSheetPanelHeight.L,
    this.barrierDismissible = true,
    this.removePill = false,
    this.dense = false,
    this.multipleItemBackgrounds = false,
  })  : child = null,
        okButtonText = null,
        useNavigator = false,
        primaryColor = null,
        cancelText = null,
        groups = null,
        _type = 2;

  DialogPanel.multipleSelectList({
    this.title,
    this.subTitle,
    required this.items,
    this.bottomSheetPanelHeight = BottomSheetPanelHeight.L,
    this.barrierDismissible = true,
    this.removePill = false,
    this.dense = false,
    this.multipleItemBackgrounds = false,
    this.okButtonText,
  })  : child = null,
        groups = null,
        useNavigator = false,
        primaryColor = null,
        cancelText = null,
        _type = 3;

  DialogPanel.form({
    this.title,
    this.subTitle,
    required this.groups,
    this.bottomSheetPanelHeight = BottomSheetPanelHeight.L,
    this.barrierDismissible = true,
    this.removePill = false,
    this.dense = false,
    this.multipleItemBackgrounds = false,
    this.okButtonText,
  })  : child = null,
        items = null,
        primaryColor = null,
        cancelText = null,
        useNavigator = false,
        _type = 4;

  @override
  Widget build(BuildContext context) {
    late Widget _current;
    if (_type == 0) {
      _current = _ChildDialogPanel(child: child!);
    }
    if (_type == 1) {
      _current = _SimpleDialogPanel(
        items: items,
        barrierDismissible: barrierDismissible,
        bottomSheetPanelHeight: bottomSheetPanelHeight,
        subTitle: subTitle,
        title: title,
      );
    }
    if (_type == 2) {
      _current = _ListDialogPanel(
        items: items!,
        barrierDismissible: barrierDismissible,
        bottomSheetPanelHeight: bottomSheetPanelHeight,
        subTitle: subTitle,
        title: title,
        dense: dense!,
        multipleItemBackgrounds: multipleItemBackgrounds!,
      );
    }
    if (_type == 3) {
      _current = _MultipleSelectListDialogPanel(
        items: items!,
        barrierDismissible: barrierDismissible,
        bottomSheetPanelHeight: bottomSheetPanelHeight,
        subTitle: subTitle,
        title: title,
        dense: dense!,
        multipleItemBackgrounds: multipleItemBackgrounds!,
        okButtonText: okButtonText,
      );
    }
    if (_type == 4) {
      _current = _FormDialogPanel(
        groups: groups!,
        barrierDismissible: barrierDismissible,
        bottomSheetPanelHeight: bottomSheetPanelHeight,
        subTitle: subTitle,
        title: title,
        dense: dense!,
        multipleItemBackgrounds: multipleItemBackgrounds!,
        okButtonText: okButtonText,
      );
    }

    if (_type == 5) {
      _current = _DefaultDialogPanel(
        subTitle: subTitle,
        title: title,
        okText: okButtonText,
        cancelText: cancelText,
        primaryColor: primaryColor,
      );
    }

    return _current;
  }

  @override
  bool barrierDismissible;
}
