import 'package:flutter/material.dart';

import '../../mcg_extension.dart';
import 'other/over.gesture_widget.dart';

class OverBottomSheetWidget extends StatefulWidget {
  OverBottomSheetWidget();

  @override
  OverBottomSheetWidgetState createState() => OverBottomSheetWidgetState();
}

class OverBottomSheetWidgetState extends State<OverBottomSheetWidget> {
  var _initialPosition = 0.0;
  var _currentPosition = 0.0;

  bool _isVisible = false;
  double? _extraOpacity;
  OverController get _overController => Get.find<OverController>();

  @override
  void initState() {
    _overController.overBottomSheetState = this;
    Future.microtask(() {
      setState(() {
        _isVisible = true;
      });
    });
    super.initState();
  }

  Future<void> close() async {
    if (mounted)
      setState(() {
        _isVisible = false;
      });
    _overController.overBottomSheetState = null;
    await Future.delayed(OverExtension.kSheetAnimationDuration);
  }

  double? deviceWidth;
  double? deviceHeight;

  @override
  Widget build(BuildContext context) {
    final mediaQuery = MediaQuery.of(context);
    deviceWidth = mediaQuery.size.width;
    deviceHeight = mediaQuery.size.height;
    double availableHeight = deviceHeight! - mediaQuery.viewInsets.bottom;
    final child = _overController.panel!;

    if (child is DialogPanel) {
      Widget _current = Padding(
        padding: EdgeInsets.only(
          bottom: mediaQuery.viewInsets.bottom,
          top: mediaQuery.padding.top,
        ),
        child: Center(
          child: Container(
            constraints: BoxConstraints(
              maxHeight: ((availableHeight < 480) ? availableHeight : availableHeight * 2 / 3) - (mediaQuery.padding.top + mediaQuery.padding.bottom + 16),
              maxWidth: deviceWidth! > 700 ? 640 : (deviceWidth! - 48.0),
            ),
            child: AnimatedOpacity(
              duration: OverExtension.kSheetAnimationDuration,
              opacity: _isVisible ? 1.0 : 0.0,
              child: Material(
                color: Fav.design.sheet.background,
                elevation: 24.0,
                type: MaterialType.card,
                child: child,
                shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(20)),
              ),
            ),
          ),
        ),
      );

      //? Barrier color acaba siyah olsa daha guzel olur. ama bottom sheeti dene cunku blur rengi verirken sacmaliyor
      final _barrierColor = Fav.design.scaffold.background;

      _current = GestureDetector(
        onTap: child.barrierDismissible ? OverBottomSheet.closeBottomSheet : null,
        child: AnimatedContainer(
          duration: OverExtension.kSheetAnimationDuration,
          color: _extraOpacity != null
              ? _barrierColor.withAlpha((120 * _extraOpacity!).toInt())
              : _isVisible
                  ? _barrierColor.withAlpha(120)
                  : _barrierColor.withAlpha(0),
          child: GestureDetector(onTap: () {}, child: _current),
        ),
      );

      if (child.useNavigator) {
        _current = HeroControllerScope(
          controller: MaterialApp.createMaterialHeroController(),
          child: Navigator(
            onPopPage: (route, result) => route.didPop(result),
            pages: [MaterialPage(child: _current)],
          ),
        );
      }

      return _current;
    } else if (child is BottomSheetPanel) {
      Widget _current = Padding(
        padding: EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom),
        child: GestureDetector(
          onTap: () {},
          child: Container(
            width: deviceWidth,
            constraints: BoxConstraints(
              maxHeight: (availableHeight < 600 ? (availableHeight / 1.075) : (child.bottomSheetPanelHeight == BottomSheetPanelHeight.M ? (availableHeight / 1.5) : (availableHeight / 1.075))) - (deviceWidth! > 600 ? 16 : 0),
              maxWidth: deviceWidth! > 700 ? 640 : deviceWidth!,
            ),
            child: AnimatedContainer(
              transform: _isVisible ? Matrix4.translationValues(0, _currentPosition, 0) : Matrix4.translationValues(0, _currentPosition + availableHeight / 3, 0),
              duration: OverExtension.kSheetAnimationDuration,
              child: AnimatedOpacity(
                duration: OverExtension.kSheetAnimationDuration,
                opacity: _isVisible ? 1.0 : 0.0,
                child: Material(
                  color: Fav.design.sheet.background,
                  elevation: 24.0,
                  type: MaterialType.card,
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      if (child.barrierDismissible && child.removePill != true) PillGesture(pillColor: Fav.design.sheet.card),
                      Flexible(child: child),
                      (context.screenBottomPadding).heightBox,
                    ],
                  ),
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(20.0),
                      topRight: Radius.circular(20.0),
                      bottomLeft: Radius.circular(deviceWidth! > 600 ? 20.0 : 0),
                      bottomRight: Radius.circular(deviceWidth! > 600 ? 20.0 : 0),
                    ),
                  ),
                ),
              ),
            ),
          ),
        ),
      );

      if (child.barrierDismissible) {
        _current = GestureDetector(
          behavior: HitTestBehavior.translucent,
          onVerticalDragStart: _onVerticalDragStart,
          onVerticalDragUpdate: _onVerticalDragUpdate,
          onVerticalDragEnd: _onVerticalDragEnd,
          child: _current,
        );
      }

      //? Barrier color acaba siyah olsa daha guzel olur. ama bottom sheeti dene cunku blur rengi verirken sacmaliyor
      final _barrierColor = Fav.design.scaffold.background;

      _current = GestureDetector(
        onTap: child.barrierDismissible ? OverBottomSheet.closeBottomSheet : null,
        child: AnimatedContainer(
          duration: OverExtension.kSheetAnimationDuration,
          color: _extraOpacity != null
              ? _barrierColor.withAlpha((120 * _extraOpacity!).toInt())
              : _isVisible
                  ? _barrierColor.withAlpha(120)
                  : _barrierColor.withAlpha(0),
          child: Column(
            children: [
              Spacer(),
              _current,
              if (deviceWidth! > 600) 16.heightBox,
            ],
          ),
        ),
      );

      if (child.useNavigator) {
        _current = HeroControllerScope(
          controller: MaterialApp.createMaterialHeroController(),
          child: Navigator(
            onPopPage: (route, result) => route.didPop(result),
            pages: [MaterialPage(child: _current)],
          ),
        );
      }

      return _current;
    }

    return Container();
  }

  void _onVerticalDragStart(DragStartDetails drag) {
    _initialPosition = drag.globalPosition.dy;
  }

  void _onVerticalDragUpdate(DragUpdateDetails drag) {
    setState(() {
      final temp = _currentPosition;
      _currentPosition = drag.globalPosition.dy - _initialPosition;

      _extraOpacity = ((_currentPosition) / (deviceHeight! - _initialPosition)).clamp(0.0, 1.0);
      _extraOpacity = 1 - _extraOpacity!;
      if (_currentPosition < 0) {
        _currentPosition = temp;
      }
    });
  }

  void _onVerticalDragEnd(DragEndDetails drag) {
    _extraOpacity = null;
    if (_currentPosition > 100.0) {
      OverBottomSheet.closeBottomSheet();
      return;
    }
    setState(() {
      _currentPosition = 0.0;
    });
  }
}
