import 'package:avatar_glow/avatar_glow.dart';
import 'package:flutter/material.dart';

import '../../mcg_extension.dart';

class OverLoadingWidget extends StatefulWidget {
  OverLoadingWidget();

  @override
  OverLoadingWidgetState createState() => OverLoadingWidgetState();
}

class OverLoadingWidgetState extends State<OverLoadingWidget> {
  bool _isVisible = false;
  OverController get _overController => Get.find<OverController>();
  @override
  void initState() {
    _overController.overloadingState = this;
    Future.microtask(() {
      setState(() {
        _isVisible = true;
      });
    });
    super.initState();
  }

  bool? closeState;
  Future<void> close({bool? state}) async {
    closeState = state;

    if (closeState != null) {
      setState(() {});
      await 2000.wait;
    }

    if (mounted)
      setState(() {
        _isVisible = false;
      });
    _overController.overloadingState = null;
    await Future.delayed(OverExtension.kSheetAnimationDuration);
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedContainer(
      duration: OverExtension.kSheetAnimationDuration,
      color: _isVisible ? Colors.black.withAlpha(120) : Colors.black.withAlpha(0),
      child: AnimatedOpacity(
        duration: OverExtension.kSheetAnimationDuration,
        opacity: _isVisible ? 1.0 : 0.0,
        child: Container(
          width: double.infinity,
          height: double.infinity,
          alignment: Alignment.center,
          child: _LoadingWidget(
            text: _overController.loadingText,
            state: closeState,
          ),
        ),
      ),
    );
  }
}

class _LoadingWidget extends StatelessWidget {
  final _kChangeStateDuration = Duration(milliseconds: 444);
  final String? text;
  final bool? state;

  _LoadingWidget({
    this.text,
    this.state,
  });

  @override
  Widget build(BuildContext context) {
    Widget current = Center(
      child: SizedBox(
        width: 48,
        height: 48,
        child: AnimatedSwitcher(
          duration: _kChangeStateDuration,
          child: state == null
              ? CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation(Colors.white),
                  strokeWidth: 2,
                )
              : state == true
                  ? Icons.done_rounded.icon.color(Colors.white).make()
                  : Icons.error_outline.icon.color(Colors.white).make(),
        ),
      ),
    );

    final _color = state == null
        ? Fav.design.primary
        : state == true
            ? Colors.green
            : Colors.redAccent;
    current = AnimatedContainer(
      duration: _kChangeStateDuration,
      width: 64,
      height: 64,
      decoration: BoxDecoration(
          boxShadow: [BoxShadow(color: _color.hue30.withAlpha(100))],
          shape: BoxShape.circle,
          gradient: LinearGradient(
            colors: [_color, _color.hue30],
            begin: Alignment.topLeft,
            end: Alignment.bottomRight,
          )),
      child: current,
    );

    current = AvatarGlow(
      child: current,
      endRadius: 48,
      startDelay: 10.milliseconds,
      duration: 4.seconds,
      glowColor: Fav.design.primaryText,
    );

    if (text != null) {
      current = Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          current,
          const SizedBox(height: 8),
          Material(
            color: Colors.transparent,
            child: Text(text!, textAlign: TextAlign.center, style: const TextStyle(color: Colors.white)),
          ),
        ],
      );
    }

    return current;
  }
}
