import 'dart:async';

import 'package:back_button_interceptor/back_button_interceptor.dart';
import 'package:flutter/material.dart';

import '../mcg_extension.dart';
import 'alert/over.alert_widget.dart';
import 'bottomsheet_dialog/bottomsheet_panels/sure.dart';
import 'bottomsheet_dialog/dialog_panels/sure.dart';
import 'bottomsheet_dialog/main_widget.dart';
import 'loading/over.loading_widget.dart';
import 'progress/over.progress_widget.dart';

part 'over.widget.dart';

class Over {
  Over._();

  static Future<bool> sure({String? message, String? title, String? cancelText, String? yesText, Color? primaryColor, bool isDialog = false}) async {
    if (isDialog) {
      final _result = await Get.find<OverController>().showBottomSheet(
        DialogPanel.child(
          child: SureDialogPanel(cancelText: cancelText, subTitle: message, title: title, yesText: yesText, primaryColor: primaryColor),
          barrierDismissible: false,
        ),
      );
      return _result ?? false;
    } else {
      final _result = await Get.find<OverController>().showBottomSheet(
        BottomSheetPanel.child(
          child: SureBottomSheetPanel(cancelText: cancelText, subTitle: message, title: title, yesText: yesText, primaryColor: primaryColor),
          barrierDismissible: false,
        ),
      );
      return _result ?? false;
    }
  }
}

class OverProgress {
  OverProgress._();

  static void setValue({required double value}) {
    assert(Get.isRegistered<OverController>());
    Get.find<OverController>().progressValue.value = value.clamp(0.0, 1.0);
  }

  static void show({String? text}) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().showProgress(text: text);
  }

  static Future<void> close() {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().closeProgress();
  }
}

class OverLoading {
  OverLoading._();

  static void show({String? text, bool scaffoldIsGoBack = false}) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().showLoading(text: text, scaffoldMustGoBack: scaffoldIsGoBack);
  }

  static Future<void> close({bool? state}) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().closeLoading(state: state);
  }
}

class OverBottomSheet {
  OverBottomSheet._();

  static Future show(BottomSheetPanel bottomSheetPanel) async {
    assert(Get.isRegistered<OverController>());
    final _result = await Get.find<OverController>().showBottomSheet(bottomSheetPanel);

    return _result;
  }

  static void closeBottomSheet() {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().closePanel();
  }

  static void selectBottomSheetItem(value) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().selectPanelItem(value);
  }
}

class OverDialog {
  OverDialog._();

  static Future show(DialogPanel dialogPanel) async {
    assert(Get.isRegistered<OverController>());
    final _result = await Get.find<OverController>().showBottomSheet(dialogPanel);
    return _result;
  }

  static void closeDialog() {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().closePanel();
  }

  static void selectDialog(value) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().selectPanelItem(value);
  }
}

class OverAlert {
  OverAlert._();

  static void saveErr() => show(message: 'saveerr'.translate, type: AlertType.danger);
  static void deleteErr() => show(type: AlertType.danger, message: 'deleteerr'.translate);
  static void saveSuc() => show(message: 'savesuc'.translate, type: AlertType.successful);
  static void deleteSuc() => show(message: 'deletesuc'.translate, type: AlertType.successful);
  static void fillRequired() => show(message: 'fillrequired'.translate, type: AlertType.warning);
  static void nothingFoundToSave() => show(message: 'nothingfoundtosave'.translate, type: AlertType.warning);
  static void anError() => show(message: 'anerror'.translate, type: AlertType.danger);

  static void show({AlertType type = AlertType.info, String? title, required String message, bool autoClose = true}) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().addAlert(OverAlertModel(type: type, message: message, autoClose: autoClose, title: title));
  }

  static OverAlertModel child(Widget child) {
    assert(Get.isRegistered<OverController>());
    final _model = OverAlertModel.child(child: child);
    Get.find<OverController>().addAlert(_model);
    return _model;
  }

//? Herhangi bir widgeti alert yapmissan kapatirken overalertmodeli yollamalisin
  static void remove(OverAlertModel model) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().removeAlert(model);
  }
}

class OverExtension {
  static const kSheetAnimationDuration = Duration(milliseconds: 200);
  static const kAlertAnimationDuration = Duration(milliseconds: 300);

  static Future<void> changeConnectionLostWidget(bool state) async {
    await 1000.wait;
    if (!Get.isRegistered<OverController>()) return;
    return Get.find<OverController>().changeConnectionLostState(state);
  }
}
