part of 'over.dart';

//Ilerde burayi getxcontroller yapip asagiyida statelesswidget yapabilirsin
class OverController extends GetxController {
  OverLoadingWidgetState? overloadingState;
  OverBottomSheetWidgetState? overBottomSheetState;
  OverProgressWidgetState? overProgressWidgetState;

  OverController({this.visibleConntectionLost = true});

  @override
  void onInit() {
    BackButtonInterceptor.add(myInterceptor);
    super.onInit();
  }

  @override
  void onClose() {
    BackButtonInterceptor.remove(myInterceptor);
    super.onClose();
  }

  //* false degeri donuyorsa back button default gorevini yapar
  bool myInterceptor(bool stopDefaultButtonEvent, RouteInfo info) {
    if (panel != null) {
      if (panel!.barrierDismissible == false) {
        return true;
      } else {
        closePanel();
      }

      return true;
    }
    if (loadingVisible.value) {
      return true;
    }
    if (progressVisible.value) {
      return true;
    }

    return false;
  }

  var scaffoldIsGoBack = false.obs;

  //* LoadinState Start
  var loadingVisible = false.obs;
  String? loadingText;
  void showLoading({String? text, bool scaffoldMustGoBack = false}) {
    if (loadingVisible.value == true) return;
    loadingText = text;
    loadingVisible.value = true;
    if (scaffoldMustGoBack == true) scaffoldIsGoBack.value = true;
  }

  Future<void> closeLoading({bool? state}) async {
    if (overloadingState == null) {
      await 75.wait;
      // ignore: invariant_booleans
      if (overloadingState == null) return;
    }
    if (scaffoldIsGoBack.value == true) scaffoldIsGoBack.value = false;
    await overloadingState!.close(state: state);
    loadingText = '';
    loadingVisible.value = false;
    await 75.wait;
  }

//* LoadinState End

  //* Progress State Start
  var progressVisible = false.obs;
  String? progressText;
  var progressValue = 0.0.obs;
  void showProgress({String? text}) {
    if (progressVisible.value == true) return;
    progressText = text;
    progressValue.value = 0.0;
    progressVisible.value = true;
    scaffoldIsGoBack.value = true;
  }

  Future<void> closeProgress() async {
    if (overProgressWidgetState == null) {
      await 75.wait;
      // ignore: invariant_booleans
      if (overProgressWidgetState == null) return;
    }
    scaffoldIsGoBack.value = false;
    await overProgressWidgetState!.close();
    progressText = '';
    progressVisible.value = false;
    await 75.wait;
  }
//* Progress State End

//* BottomSheet & Dialog panels
  IPanel? panel;
  Completer? _panelCompleter;
  var panelIsVisible = false.obs;
  Future showBottomSheet(IPanel child) async {
    if (panel != null || _panelCompleter != null) throw ('BottomSheetBusy');

    _panelCompleter = Completer();
    panel = child;
    panelIsVisible.value = true;
    scaffoldIsGoBack.value = true;

    final _result = await _panelCompleter!.future;

    scaffoldIsGoBack.value = false;
    await overBottomSheetState!.close();
    panelIsVisible.value = false;
    panel = null;
    _panelCompleter = null;
    await 15.wait;
    return _result;
  }

  void closePanel() {
    if (_panelCompleter == null || _panelCompleter!.isCompleted == true) return;
    _panelCompleter!.complete(null);
  }

  void selectPanelItem(value) {
    if (_panelCompleter == null || _panelCompleter!.isCompleted == true) return;
    _panelCompleter!.complete(value);
  }

//* OverBottomSheet End

//*ConnectionState Start
  bool visibleConntectionLost = true;
  var connectionLostVisible = false.obs;
  void changeConnectionLostState(bool state) {
    if (visibleConntectionLost == false) return;
    connectionLostVisible.value = state;
  }

//*ConnectionState End

//* Alert start
  final _animatedListKey = GlobalKey<AnimatedListState>();
  final List<OverAlertModel> _alertModelList = [];
  void addAlert(OverAlertModel model) {
    _alertModelList.add(model);
    _animatedListKey.currentState!.insertItem(_alertModelList.length - 1, duration: OverExtension.kAlertAnimationDuration);

    if (model.autoClose == true) {
      Future.delayed(Duration(seconds: 3)).then((value) => removeAlert(model));
    }
  }

  void removeAlert(OverAlertModel model) {
    if (!_alertModelList.contains(model)) return;

    _animatedListKey.currentState!.removeItem(_alertModelList.indexOf(model), (_, animation) {
      return SlideTransition(
        position: Tween<Offset>(
          begin: const Offset(1, 0),
          end: const Offset(0, 0),
        ).animate(animation),
        child: OverAlertWidget(model),
      );
    }, duration: OverExtension.kAlertAnimationDuration);

    _alertModelList.remove(model);
  }

//* Alert end
}

class OverWidget extends StatelessWidget {
  final bool visibleConntectionLost;
  final Widget child;

  OverWidget({required this.child, this.visibleConntectionLost = true});

  @override
  Widget build(BuildContext context) {
    final _overController = Get.findOrPut<OverController>(createFunction: () => OverController(visibleConntectionLost: visibleConntectionLost), permanent: true);
    Widget _childWidget = Obx(() {
      return AnimatedContainer(
        transformAlignment: Alignment.center,
        duration: OverExtension.kSheetAnimationDuration,
        decoration: BoxDecoration(borderRadius: BorderRadius.circular(_overController.scaffoldIsGoBack.value ? 12.0 : 0.0)),
        clipBehavior: Clip.antiAlias,
        transform: (_overController.scaffoldIsGoBack.value ? (Matrix4.identity()..scale(0.9, 0.9)) : Matrix4.identity()),
        child: child,
      );
    });
    Widget _currrent = Material(
      ///Burda siyah renk verildiginde backdropfilter yanlis calisiyordu bottomsheet te sikinti yoksa acabilirsin.
      color: Colors.transparent,
      child: Stack(
        children: [
          Positioned.fill(child: _childWidget),
          Obx(() => !_overController.connectionLostVisible.value
              ? SizedBox()
              : Positioned(
                  left: 0,
                  right: 0,
                  bottom: 0,
                  child: Container(
                    padding: EdgeInsets.only(bottom: context.screenBottomPadding),
                    color: Colors.redAccent,
                    child: 'connectionlost'.translate.text.color(Colors.white).fontSize(10).center.make(),
                  ),
                )),
          Positioned.fill(child: Obx(() {
            return _overController.panelIsVisible.value ? OverBottomSheetWidget() : SizedBox();
          })),
          Positioned.fill(child: Obx(() {
            return _overController.loadingVisible.value ? OverLoadingWidget() : SizedBox();
          })),
          Positioned.fill(child: Obx(() {
            return _overController.progressVisible.value ? OverProgressWidget() : SizedBox();
          })),
          Positioned(
            right: 0,
            top: context.screenTopPadding,
            left: context.screenWidth > 600 ? (context.screenWidth - 460) : 0,
            child: AnimatedList(
              key: _overController._animatedListKey,
              initialItemCount: 0,
              padding: const EdgeInsets.all(0),
              shrinkWrap: true,
              itemBuilder: (_, index, animation) {
                final _item = _overController._alertModelList[index];
                return SizeTransition(
                  key: ObjectKey(_item),
                  sizeFactor: animation,
                  child: OverAlertWidget(_item),
                );
              },
            ),
          )
        ],
      ),
    );

    return _currrent;
  }
}
