part of 'over_page.dart';

class OPTopBar {
  final VoidCallback? backButtonPressed;
  final Widget? middle;
  List<Widget>? trailingActions;
  List<Widget>? leadingActions;
  final bool hideBackButton;

  final Widget? child;

  OPTopBar({this.backButtonPressed, this.middle, this.trailingActions, this.leadingActions, this.hideBackButton = false}) : child = null;
  OPTopBar.oneChild({this.child})
      : trailingActions = null,
        leadingActions = null,
        middle = null,
        backButtonPressed = null,
        hideBackButton = false;

  void addTrailingActions(Widget item) {
    trailingActions ??= [];
    trailingActions!.add(item);
  }
}

abstract class _ITopActions {
  final bool pinned;

  //Widget get bigWidget;
  Widget get miniWidget;
  Widget get pinnedWidget;
  Widget get nonPinnedWidget;

  _ITopActions({this.pinned = false});
}

class OPTopActions extends _ITopActions {
  final Widget child;
  final Widget? miniChild;

  OPTopActions({required this.child, this.miniChild, bool pinned = false}) : super(pinned: pinned);

  // @override
  // Widget get bigWidget => child;

  @override
  Widget get miniWidget => miniChild ?? SizedBox();

  @override
  Widget get pinnedWidget => pinned ? child : SizedBox();

  @override
  Widget get nonPinnedWidget => pinned ? SizedBox() : child;
}

class OPTopActionsTitle extends _ITopActions {
  final String title;
  final Color? color;
  final IconData? iconData;
  final String? imgUrl;
  OPTopActionsTitle({required this.title, this.color, this.iconData, this.imgUrl}) : super();

  @override
  Widget get miniWidget => imgUrl != null || iconData != null
      ? Row(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            if (imgUrl != null) Image.asset(imgUrl!, height: 24),
            2.widthBox,
            if (iconData != null) iconData!.icon.color(color ?? Fav.design.primary).padding(0).make(),
            title.text.color(color ?? Fav.design.primary).autoSize.maxLines(1).bold.make(),
          ],
        )
      : title.text.color(color ?? Fav.design.primary).autoSize.maxLines(1).bold.make();

  @override
  Widget get pinnedWidget => SizedBox();

  @override
  Widget get nonPinnedWidget => Padding(
        padding: EdgeInsets.symmetric(horizontal: Get.context!.screenLeftPadding + 14, vertical: 5),
        child: imgUrl != null || iconData != null
            ? Row(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  if (imgUrl != null) Image.asset(imgUrl!, height: 34),
                  if (iconData != null) iconData!.icon.color(color ?? Fav.design.primary).size(28).padding(0).make(),
                  6.widthBox,
                  title.text.color(color ?? Fav.design.primary).fontSize(34).autoSize.maxLines(1).bold.make(),
                ],
              )
            : title.text.color(color ?? Fav.design.primary).fontSize(34).autoSize.maxLines(1).bold.make(),
      );
}

class OPTopActionsTitleWithChild extends _ITopActions {
  final OPTopActionsTitle title;
  final Widget child;

  OPTopActionsTitleWithChild({required this.title, bool childIsPinned = true, required this.child}) : super(pinned: childIsPinned);

  @override
  Widget get miniWidget => title.miniWidget;

  @override
  Widget get pinnedWidget => pinned
      ? Padding(
          padding: EdgeInsets.only(left: Get.context!.screenLeftPadding + 14, right: Get.context!.screenLeftPadding + 14, bottom: 6),
          child: Center(child: child),
        )
      : SizedBox();

  @override
  Widget get nonPinnedWidget => Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          title.nonPinnedWidget,
          // Container(
          //   height: 1,
          //   color: Fav.design.primaryText.withAlpha(10),
          //   margin: const EdgeInsets.only(top: 0, bottom: 0),
          // ),
          if (!pinned)
            Padding(
              padding: EdgeInsets.only(top: 6, left: Get.context!.screenLeftPadding + 14, right: Get.context!.screenLeftPadding + 14, bottom: 6),
              child: Center(child: child),
            ),
        ],
      );
}

class OPBottomBar {
  final Widget child;

  OPBottomBar({required this.child});

  OPBottomBar.saveButton({required Function() onPressed, required bool isLoading})
      : child = Align(
          alignment: Alignment.centerRight,
          child: OPProgressButton(
            label: 'save'.translate,
            onPressed: onPressed,
            isLoading: isLoading,
          ),
        ).p8;

  OPBottomBar.row({required List<Widget> children, MainAxisAlignment mainAxisAlignment = MainAxisAlignment.end})
      : child = Row(
          children: children,
          mainAxisAlignment: mainAxisAlignment,
          crossAxisAlignment: CrossAxisAlignment.center,
        ).px8;
}

class OPBody {
  final bool withKeyboardCloserGesture;
  final Widget? child;
  final Widget? singleChildScroll;
  final Widget Function(BuildContext context, int index)? itemBuilder;
  final Function(int, int)? onReorder;
  final int? itemCount;
  final int? crossAxisCount;
  final double? maxWidth;
  final ScrollController? scrollController;

  ///Simdilik sadece responsivescaffolda listview de ayarlandi
  final Widget? listviewFirstWidget;
  final String? pageStorageKey;

  final EdgeInsets? padding;

  OPBody({
    this.child,
    this.singleChildScroll,
    this.itemCount,
    this.itemBuilder,
    this.maxWidth,
    this.crossAxisCount,
    this.scrollController,
    this.withKeyboardCloserGesture = false,
    this.listviewFirstWidget,
    this.pageStorageKey,
    this.padding,
  }) : onReorder = null;

  OPBody.circularProgressIndicator()
      : child = Center(
            child: CircularProgressIndicator(
          valueColor: AlwaysStoppedAnimation(Fav.design.progressIndicator.indicator),
        )),
        itemBuilder = null,
        itemCount = null,
        singleChildScroll = null,
        listviewFirstWidget = null,
        maxWidth = null,
        pageStorageKey = null,
        scrollController = null,
        withKeyboardCloserGesture = true,
        padding = null,
        crossAxisCount = null,
        onReorder = null;

  OPBody.singleChildScrollView({required Widget child, this.maxWidth, this.withKeyboardCloserGesture = false, this.scrollController, this.pageStorageKey, this.padding})
      : child = null,
        itemBuilder = null,
        itemCount = null,
        singleChildScroll = child,
        listviewFirstWidget = null,
        crossAxisCount = null,
        onReorder = null;
  OPBody.child({required this.child, this.maxWidth, this.withKeyboardCloserGesture = false, this.padding})
      : singleChildScroll = null,
        itemBuilder = null,
        itemCount = null,
        scrollController = null,
        pageStorageKey = null,
        listviewFirstWidget = null,
        crossAxisCount = null,
        onReorder = null;
  OPBody.listviewBuilder({required this.itemBuilder, required this.itemCount, this.maxWidth, this.withKeyboardCloserGesture = false, this.listviewFirstWidget, this.scrollController, this.pageStorageKey, this.padding})
      : child = null,
        singleChildScroll = null,
        crossAxisCount = null,
        onReorder = null;
  OPBody.gridviewBuilder({required this.itemBuilder, required this.itemCount, this.crossAxisCount, this.maxWidth, this.withKeyboardCloserGesture = false, this.scrollController, this.pageStorageKey, this.padding})
      : child = null,
        listviewFirstWidget = null,
        singleChildScroll = null,
        onReorder = null;

  OPBody.reorderableListviewBuilder({required this.itemBuilder, required this.onReorder, required this.itemCount, this.maxWidth, this.withKeyboardCloserGesture = false, this.scrollController, this.pageStorageKey, this.padding})
      : child = null,
        listviewFirstWidget = null,
        crossAxisCount = null,
        singleChildScroll = null;

  bool get isScrollable => itemBuilder != null || singleChildScroll != null;
}

class OPDrawerPanel {
  final Drawer drawer;
  final bool isRightDrawer;
  OPDrawerPanel({required this.drawer, this.isRightDrawer = false});

  Drawer? get leftDrawer => isRightDrawer ? null : drawer;
  Drawer? get rightDrawer => isRightDrawer ? drawer : null;
}

class OPAppScaffold extends StatefulWidget {
  final OPTopBar topBar;
  final _ITopActions? topActions;
  final OPBottomBar? bottomBar;
  final OPBody body;
  final FloatingActionButton? floatingActionButton;
  final OPDrawerPanel? drawerPanel;
  final Key? scaffoldKey;
  final Color? scaffoldBackgroundColor;
  final double? blurVisiblePercentage;
  final Widget? backgroundDecoration;

  OPAppScaffold({this.topActions, this.bottomBar, required this.topBar, required this.body, this.floatingActionButton, this.drawerPanel, this.scaffoldKey, this.scaffoldBackgroundColor, Key? key, this.blurVisiblePercentage, this.backgroundDecoration}) : super(key: key);

  @override
  State<OPAppScaffold> createState() => _OPAppScaffoldState();
}

class _OPAppScaffoldState extends State<OPAppScaffold> {
  final double _kNavigationBarHeight = 44;

  late ScrollController _scrollController;

  double _bottomBarHeight = 0.0;
  // double _topActionsHeight = 0.0;
  double get _topActionsMaxHeight => _topActionsMaxHeightPinnedWidget + _topActionsMaxHeightNonPinnedWidget;

  double _topActionsMaxHeightPinnedWidget = 0.0;
  double _topActionsMaxHeightNonPinnedWidget = 0.0;

  bool _topActionsVisible = true;

  @override
  void initState() {
    _scrollController = widget.body.scrollController ?? ScrollController();

    super.initState();
  }

  int? _lastBodyHash;
  void _baziDegisikliklerAlgilandigindaYapilacaklar() {
    _topActionsVisible = true;
  }

  @override
  Widget build(BuildContext context) {
    if (_lastBodyHash == null) {
      _lastBodyHash = widget.body.hashCode;
    } else if (_lastBodyHash != widget.body.hashCode) {
      _baziDegisikliklerAlgilandigindaYapilacaklar();
      _lastBodyHash = widget.body.hashCode;
    }

    var _leftPadding = widget.body.padding?.left ?? 0.0;
    var _rightPadding = widget.body.padding?.right ?? 0.0;

    if (widget.body.maxWidth != null && context.screenWidth > widget.body.maxWidth!) {
      _leftPadding += (context.screenWidth - widget.body.maxWidth!) / 2;
      _rightPadding += _leftPadding;
    }

    final _bottomPadding = widget.body.padding?.bottom;
    final _topPadding = widget.body.padding?.top;

    Widget _body = widget.body.child != null
        ? AnimatedPadding(
            duration: 222.milliseconds,
            padding: EdgeInsets.only(
              bottom: widget.bottomBar != null ? (_bottomBarHeight + context.screenBottomPadding + (_bottomPadding ?? 0)) : (_bottomPadding ?? 0),
              top: _kNavigationBarHeight + _topActionsMaxHeight + (_topPadding ?? 0),
              left: _leftPadding,
              right: _rightPadding,
            ),
            child: widget.body.child,
          )
        : widget.body.singleChildScroll != null
            ? SingleChildScrollView(
                key: widget.body.pageStorageKey == null ? null : PageStorageKey(widget.body.pageStorageKey),
                controller: _scrollController,
                child: widget.body.singleChildScroll,
                physics: const BouncingScrollPhysics(),
                padding: EdgeInsets.only(
                  left: _leftPadding + 8,
                  right: _rightPadding + 8,
                  bottom: widget.bottomBar != null ? _bottomBarHeight + context.screenBottomPadding + (_bottomPadding ?? 8) : (_bottomPadding ?? 8),
                  top: _kNavigationBarHeight + _topActionsMaxHeight + (_topPadding ?? 8),
                ),
              ).px8
            : widget.body.onReorder != null
                ? ReorderableListView.builder(
                    shrinkWrap: true,
                    key: widget.body.pageStorageKey == null ? null : PageStorageKey(widget.body.pageStorageKey),
                    scrollController: _scrollController,
                    itemBuilder: widget.body.itemBuilder!,
                    itemCount: widget.body.itemCount!,
                    physics: const BouncingScrollPhysics(),
                    onReorder: widget.body.onReorder!,
                    padding: EdgeInsets.only(
                      left: _leftPadding + 8,
                      right: _rightPadding + 8,
                      bottom: widget.bottomBar != null ? _bottomBarHeight + context.screenBottomPadding + (_bottomPadding ?? 8) : (_bottomPadding ?? 8),
                      top: _kNavigationBarHeight + _topActionsMaxHeight + (_topPadding ?? 8),
                    ),
                  )
                : widget.body.itemBuilder != null && widget.body.crossAxisCount != null
                    ? GridView.builder(
                        shrinkWrap: true,
                        key: widget.body.pageStorageKey == null ? null : PageStorageKey(widget.body.pageStorageKey),
                        controller: _scrollController,
                        itemBuilder: widget.body.itemBuilder!,
                        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: widget.body.crossAxisCount!),
                        itemCount: widget.body.itemCount,
                        physics: const BouncingScrollPhysics(),
                        padding: EdgeInsets.only(
                          left: _leftPadding + 8,
                          right: _rightPadding + 8,
                          bottom: widget.bottomBar != null ? _bottomBarHeight + context.screenBottomPadding + (_bottomPadding ?? 8) : (_bottomPadding ?? 8),
                          top: _kNavigationBarHeight + _topActionsMaxHeight + (_topPadding ?? 8),
                        ),
                      )
                    : widget.body.itemBuilder != null
                        ? ListView.builder(
                            shrinkWrap: true,
                            key: widget.body.pageStorageKey == null ? null : PageStorageKey(widget.body.pageStorageKey),
                            controller: _scrollController,
                            itemBuilder: widget.body.itemBuilder!,
                            itemCount: widget.body.itemCount,
                            physics: const BouncingScrollPhysics(),
                            padding: EdgeInsets.only(
                              left: _leftPadding + 8,
                              right: _rightPadding + 8,
                              bottom: widget.bottomBar != null ? _bottomBarHeight + context.screenBottomPadding + (_bottomPadding ?? 8) : (_bottomPadding ?? 8),
                              top: _kNavigationBarHeight + _topActionsMaxHeight + (_topPadding ?? 8),
                            ),
                          )
                        : const SizedBox();

    if (widget.body.isScrollable && (widget.topActions?.pinned == false || widget.topActions is OPTopActionsTitleWithChild)) {
      _body = NotificationListener<UserScrollNotification>(
        onNotification: (notification) {
          final _direction = notification.direction;
          bool? _newTopActionsVisible;
          if (_direction == ScrollDirection.reverse) {
            _newTopActionsVisible = false;
          } else if (_direction == ScrollDirection.forward) {
            _newTopActionsVisible = true;
          }
          if (_newTopActionsVisible != null && _topActionsVisible != _newTopActionsVisible) {
            setState(() {
              _topActionsVisible = _newTopActionsVisible!;
            });
          }
          return true;
        },
        child: _body,
      );
    }

    // if (widget.body.withKeyboardCloserGesture && !kIsWeb) {
    //   _body = _body.withKeyboardCloseGesture(context);
    // }

    Widget _scaffold = Scaffold(
      key: widget.scaffoldKey,
      floatingActionButton: widget.floatingActionButton,
      backgroundColor: widget.scaffoldBackgroundColor ?? Fav.design.sheet.background,
      drawer: widget.drawerPanel?.leftDrawer,
      endDrawer: widget.drawerPanel?.rightDrawer,
      body: Stack(
        children: [
          if (widget.backgroundDecoration != null) Positioned.fill(child: widget.backgroundDecoration!),
          Positioned.fill(child: _body),
          Positioned(
            top: 0,
            left: 0,
            right: 0,
            child: _ScaffoldBackDropFilter(
              backgroundColor: Fav.design.sheet.background,
              blurVisiblePercentage: widget.blurVisiblePercentage ?? 0.1,
              child: Column(
                children: [
                  Container(
                    margin: EdgeInsets.only(top: 0, left: context.screenLeftPadding, right: context.screenRightPadding),
                    alignment: Alignment.center,
                    height: _kNavigationBarHeight,
                    child: widget.topBar.child ??
                        Row(
                          mainAxisSize: MainAxisSize.max,
                          children: [
                            Expanded(
                                flex: 10,
                                child: Row(mainAxisAlignment: MainAxisAlignment.start, children: [
                                  if (widget.topBar.leadingActions != null) ...widget.topBar.leadingActions!,
                                ]).px16),

                            if (widget.topBar.middle != null)
                              Expanded(flex: 10, child: Align(alignment: Alignment.center, child: widget.topBar.middle))
                            else if (widget.topActions?.miniWidget != null)
                              Expanded(
                                flex: 10,
                                child: AnimatedSwitcher(
                                    switchInCurve: Curves.easeOut,
                                    switchOutCurve: Curves.easeIn,
                                    duration: const Duration(milliseconds: 444),
                                    reverseDuration: const Duration(milliseconds: 222),
                                    transitionBuilder: (child, animation) => FadeTransition(
                                          opacity: animation,
                                          child: child,
                                        ),
                                    child: _topActionsVisible ? const SizedBox() : Container(constraints: BoxConstraints(maxWidth: context.width / 3), alignment: Alignment.center, child: widget.topActions!.miniWidget)),
                              ),
                            //   if (widget.topBar.middle != null || widget.topActions?.miniWidget != null)
                            Expanded(
                                flex: 10,
                                child: Row(mainAxisAlignment: MainAxisAlignment.end, children: [
                                  if (widget.topBar.trailingActions != null) ...widget.topBar.trailingActions!,
                                  Icons.close_rounded.icon
                                      .color(Fav.design.sheet.itemText)
                                      .padding(0)
                                      .size(24)
                                      .onPressed(() {
                                        if (widget.topBar.backButtonPressed != null) {
                                          widget.topBar.backButtonPressed!();
                                        } else {
                                          OverBottomSheet.closeBottomSheet();
                                        }
                                      })
                                      .make()
                                      .circleBackground(background: Fav.design.sheet.itemText.withAlpha(10))
                                      .pl8,
                                ]).pr16),
                          ],
                        ),
                  ),
                  if (widget.topActions != null)
                    AnimatedSwitcher(
                        duration: const Duration(milliseconds: 222),
                        transitionBuilder: (child, animation) {
                          return SizeTransition(sizeFactor: animation, child: child);
                        },
                        child: !_topActionsVisible
                            ? SizedBox()
                            : WidgetSize(
                                onSizeChange: (size) {
                                  if (size == null) return;
                                  if (size.height != _topActionsMaxHeightNonPinnedWidget) {
                                    setState(() {
                                      _topActionsMaxHeightNonPinnedWidget = size.height;
                                    });
                                  }
                                },
                                child: widget.topActions!.nonPinnedWidget)),
                  if (widget.topActions != null)
                    WidgetSize(
                        onSizeChange: (size) {
                          if (size == null) return;
                          if (size.height != _topActionsMaxHeightPinnedWidget) {
                            setState(() {
                              _topActionsMaxHeightPinnedWidget = size.height;
                            });
                          }
                        },
                        child: widget.topActions!.pinnedWidget),
                  Container(color: Fav.design.primaryText.withAlpha(3), height: 1),
                ],
              ),
            ),
          ),
          if (widget.bottomBar != null)
            Positioned(
              left: 0,
              bottom: 0,
              right: 0,
              child: _ScaffoldBackDropFilter(
                backgroundColor: (Fav.design.sheet.background),
                blurVisiblePercentage: widget.blurVisiblePercentage ?? 0.4,
                child: WidgetSize(
                  onSizeChange: (size) {
                    if (size == null) return;
                    setState(() {
                      _bottomBarHeight = size.height - context.screenBottomPadding;
                    });
                  },
                  child: Padding(
                    padding: EdgeInsets.only(bottom: context.screenBottomPadding + 8, top: 8, right: 8, left: 8),
                    child: widget.bottomBar!.child,
                  ),
                ),
              ),
            ),
        ],
      ),
    );

    if (widget.body.child == null) {
      _scaffold = Scrollbar(
        controller: _scrollController,
        thumbVisibility: kIsWeb,
        child: _scaffold,
      );
    }

    if (widget.body.withKeyboardCloserGesture && !kIsWeb) {
      _scaffold = _scaffold.withKeyboardCloseGesture(context);
    }

    return _scaffold;
  }
}

class _ScaffoldBackDropFilter extends StatelessWidget {
  final Widget child;
  final Color backgroundColor;
  final double blurVisiblePercentage;
  _ScaffoldBackDropFilter({required this.child, required this.backgroundColor, this.blurVisiblePercentage = 0.4});
  @override
  Widget build(BuildContext context) {
    Widget _current = Container(
      color: backgroundColor.withOpacity(blurVisiblePercentage),
      child: child,
    );
    if (blurVisiblePercentage >= 1.0 || blurVisiblePercentage < 0.0) return _current;

    return ClipRRect(
      child: BackdropFilter(
        filter: ImageFilter.blur(sigmaX: 10, sigmaY: 10),
        child: _current,
      ),
    );
  }
}

class OPProgressButton extends StatelessWidget {
  final bool isLoading;
  final Function()? onPressed;
  final String label;
  final Color? color;
  final Color textColor;
  final FontWeight? fontWeight;
  final bool mini;

  OPProgressButton({this.isLoading = false, required this.onPressed, required this.label, this.color, this.mini = false, this.textColor = Colors.white, this.fontWeight});

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: mini ? 24.0 : 36.0,
      child: Material(
        elevation: 3,
        shape: const StadiumBorder(),
        color: Colors.transparent,
        type: MaterialType.button,
        child: Ink(
          decoration: BoxDecoration(
            gradient: LinearGradient(colors: [
              color ?? Fav.design.elevatedButton.variantBackground,
              (color ?? Fav.design.elevatedButton.variantBackground).hue15,
            ]),
            borderRadius: BorderRadius.circular(9),
          ),
          child: InkWell(
            customBorder: RoundedRectangleBorder(borderRadius: BorderRadius.circular(9)),
            onTap: isLoading ? null : onPressed,
            child: AnimatedContainer(
              decoration: ShapeDecoration(
                shape: isLoading ? const CircleBorder() : const StadiumBorder(),
                color: isLoading ? (color ?? Fav.design.elevatedButton.variantBackground) : Colors.transparent,
              ),
              duration: const Duration(milliseconds: 400),
              width: isLoading ? (mini ? 24.0 : 36.0) : (label.length * (mini ? 6.0 : 12.0) + (mini ? 15.0 : 22.0)),
              alignment: Alignment.center,
              child: isLoading
                  ? SizedBox(
                      width: mini ? 18.0 : 30.0,
                      height: mini ? 18.0 : 30.0,
                      child: CircularProgressIndicator(
                        strokeWidth: 3,
                        valueColor: AlwaysStoppedAnimation(textColor),
                      ))
                  : Text(label, style: TextStyle(color: textColor, fontSize: mini ? 12.0 : 16.0, fontWeight: fontWeight)),
            ),
          ),
        ),
      ),
    );
  }
}
