import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

import '../../mcg_extension.dart';

part 'mini_scaffold.dart';

class OverPage {
  OverPage._();

  static void close() {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().closePanel();
  }

  static void select(value) {
    assert(Get.isRegistered<OverController>());
    return Get.find<OverController>().selectPanelItem(value);
  }

  static void openModelBottomWithListView({
    required Widget Function(BuildContext, int) itemBuilder,
    required int itemCount,
    required String title,
    Widget? extraWidget,
    Widget? bottomBar,
    final List<Widget>? trailingActions,
    final List<Widget>? leadingActions,
    BottomSheetPanelHeight bottomSheetPanelHeight = BottomSheetPanelHeight.M,
    bool isDialog = false,
    final double? maxWidth,
  }) {
    final Widget _current = ClipRRect(
      borderRadius: BorderRadius.only(
        topLeft: Radius.circular(20.0),
        topRight: Radius.circular(20.0),
        bottomLeft: Radius.circular(Get.context!.size!.width > 600 ? 20.0 : 0),
        bottomRight: Radius.circular(Get.context!.size!.width > 600 ? 20.0 : 0),
      ),
      child: _QuickPage(
        leadingActions: leadingActions,
        trailingActions: trailingActions,
        maxHeight: bottomSheetPanelHeight == BottomSheetPanelHeight.M ? (Get.context!.size!.height / 1.5) : null,
        body: OPBody.listviewBuilder(
          itemBuilder: itemBuilder,
          itemCount: itemCount,
        ),
        title: title,
        extraWidget: extraWidget,
        bottomBar: bottomBar,
        maxWidth: maxWidth,
      ),
    );
    if (isDialog) {
      OverDialog.show(DialogPanel.child(
        child: _current,
        barrierDismissible: true,
        removePill: true,
      ));
    } else {
      OverBottomSheet.show(BottomSheetPanel.child(
        child: _current,
        barrierDismissible: true,
        removePill: true,
      ));
    }
  }

//? Map icersinde key value yolla yeter
  static Future openChoosebleListViewFromMap({
    /// dynamic,string
    required Map data,
    required String title,
    Widget? extraWidget,
    Widget? bottomBar,
    final List<Widget>? trailingActions,
    final List<Widget>? leadingActions,
    BottomSheetPanelHeight bottomSheetPanelHeight = BottomSheetPanelHeight.M,
    bool isDialog = false,
    final double? maxWidth,
    bool listNotChooseble = false,
  }) {
    final _entries = data.entries.toList();

    final Widget _current = ClipRRect(
      borderRadius: BorderRadius.only(
        topLeft: Radius.circular(20.0),
        topRight: Radius.circular(20.0),
        bottomLeft: Radius.circular(Get.context!.size!.width > 600 ? 20.0 : 0),
        bottomRight: Radius.circular(Get.context!.size!.width > 600 ? 20.0 : 0),
      ),
      child: _QuickPage(
        leadingActions: leadingActions,
        trailingActions: trailingActions,
        maxHeight: bottomSheetPanelHeight == BottomSheetPanelHeight.M ? (Get.context!.size!.height / 1.5) : null,
        body: OPBody.listviewBuilder(
          itemBuilder: (context, index) {
            final _item = _entries[index];
            return Container(
              color: index.isEven ? Colors.transparent : Fav.design.primaryText.withAlpha(4),
              child: ListTile(
                onTap: () {
                  if (listNotChooseble == false) {
                    OverBottomSheet.selectBottomSheetItem(_item.key);
                  }
                },
                title: Center(child: _item.value.toString().text.make()),
              ),
            );
          },
          itemCount: _entries.length,
        ),
        title: title,
        extraWidget: extraWidget,
        bottomBar: bottomBar,
        maxWidth: maxWidth,
      ),
    );

    if (isDialog) {
      return OverDialog.show(DialogPanel.child(
        barrierDismissible: true,
        removePill: true,
        child: _current,
      ));
    }

    return OverBottomSheet.show(BottomSheetPanel.child(
      barrierDismissible: true,
      removePill: true,
      child: _current,
    ));
  }

//? Map icersinde key value yolla yeter
  static Future openMultipleChoosebleListViewFromMap({
    /// dynamic,string
    required Map data,
    required String title,
    Widget? extraWidget,
    List<Widget>? trailingActions,
    final List<Widget>? leadingActions,
    final bool showSelectAllItemIcon = false,
    BottomSheetPanelHeight bottomSheetPanelHeight = BottomSheetPanelHeight.M,
    bool isDialog = false,
    final double? maxWidth,
  }) async {
    final _entries = data.entries.toList();
    if (Get.isRegistered<OpenMultipleChoosebleListViewFromMapController>()) await Get.delete<OpenMultipleChoosebleListViewFromMapController>();
    final _controller = Get.put<OpenMultipleChoosebleListViewFromMapController>(OpenMultipleChoosebleListViewFromMapController());

    if (showSelectAllItemIcon) {
      trailingActions ??= [];
      trailingActions.addAll([
        Icons.deselect.icon.padding(4).color(Fav.design.appBar.text).onPressed(() {
          _controller.resultList.clear();
        }).make(),
        8.widthBox,
        Icons.select_all.icon.padding(4).color(Fav.design.appBar.text).onPressed(() {
          _controller.resultList.addAll(data.keys.toList());
        }).make(),
        8.widthBox,
      ]);
    }

    final Widget _current = ClipRRect(
      borderRadius: BorderRadius.only(
        topLeft: Radius.circular(20.0),
        topRight: Radius.circular(20.0),
        bottomLeft: Radius.circular(Get.context!.size!.width > 600 ? 20.0 : 0),
        bottomRight: Radius.circular(Get.context!.size!.width > 600 ? 20.0 : 0),
      ),
      child: _QuickPage(
        leadingActions: leadingActions,
        trailingActions: trailingActions,
        maxHeight: bottomSheetPanelHeight == BottomSheetPanelHeight.M ? (Get.context!.size!.height / 1.5) : null,
        body: OPBody.listviewBuilder(
          itemBuilder: (context, index) {
            final _item = _entries[index];
            return Container(
              color: index.isEven ? Colors.transparent : Fav.design.primaryText.withAlpha(4),
              child: Obx(
                () => CheckboxListTile(
                  activeColor: Fav.design.checkBox.checked,
                  controlAffinity: ListTileControlAffinity.leading,
                  onChanged: (value) {
                    value == true ? _controller.resultList.add(_item.key) : _controller.resultList.remove(_item.key);
                  },
                  value: _controller.resultList.contains(_item.key),
                  title: Center(child: _item.value.toString().text.make()),
                ),
              ),
            );
          },
          itemCount: _entries.length,
        ),
        title: title,
        extraWidget: Obx(() => Column(
              children: [
                if (extraWidget != null) extraWidget,
                Align(
                  alignment: Alignment.topRight,
                  child: Text(
                    _controller.resultList.length.toString() + ' ' + 'multiselecthint'.translate,
                    style: TextStyle(color: Fav.design.primaryText),
                  ).stadium(background: Fav.design.primaryText.withAlpha(10)),
                ),
              ],
            )),
        saveText: 'ok'.translate,
        onSave: () {
          OverBottomSheet.selectBottomSheetItem(_controller.resultList.map((element) => element).toList());
          if (Get.isRegistered<OpenMultipleChoosebleListViewFromMapController>()) Get.delete<OpenMultipleChoosebleListViewFromMapController>();
        },
        maxWidth: maxWidth,
      ),
    );

    if (isDialog) {
      return OverDialog.show(DialogPanel.child(
        removePill: true,
        child: _current,
      ));
    }
    return OverBottomSheet.show(BottomSheetPanel.child(
      removePill: true,
      child: _current,
    ));
  }
}

class OpenMultipleChoosebleListViewFromMapController extends GetxController {
  final resultList = <dynamic>{}.obs;
}

class _QuickPage extends StatefulWidget {
  final Widget? child;
  final String title;
  final Function()? onSave;
  final String? saveText;
  final double? maxWidth;
  final Widget? extraWidget;
  final Widget? bottomBar;
  final OPBody? body;
  final double? maxHeight;
  final List<Widget>? trailingActions;
  final List<Widget>? leadingActions;

  _QuickPage({this.child, required this.title, this.onSave, this.saveText, this.maxWidth, this.extraWidget, this.bottomBar, this.body, this.maxHeight, this.trailingActions, this.leadingActions});

  @override
  State<_QuickPage> createState() => _QuickPageState();
}

class _QuickPageState extends State<_QuickPage> {
  bool _isLoading = false;

  @override
  void initState() {
    assert(widget.body != null || widget.child != null);

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    Widget _current = OPAppScaffold(
      topBar: OPTopBar(trailingActions: widget.trailingActions, leadingActions: widget.leadingActions),
      topActions: OPTopActionsTitleWithChild(
        title: OPTopActionsTitle(title: widget.title),
        child: widget.extraWidget ?? SizedBox(),
      ),
      body: widget.body ?? OPBody.singleChildScrollView(child: widget.child ?? SizedBox(), maxWidth: widget.maxWidth, withKeyboardCloserGesture: true),
      bottomBar: widget.onSave != null
          ? OPBottomBar(
              child: Align(
              alignment: Alignment.centerRight,
              child: OPProgressButton(
                label: widget.saveText ?? 'save'.translate,
                onPressed: () async {
                  setState(() {
                    _isLoading = true;
                  });
                  await widget.onSave!();
                  setState(() {
                    _isLoading = false;
                  });
                },
                isLoading: _isLoading,
              ).pr16,
            ))
          : widget.bottomBar != null
              ? OPBottomBar(child: widget.bottomBar!)
              : null,
    );

    if (widget.maxHeight != null) {
      _current = SizedBox(
        height: widget.maxHeight,
        child: _current,
      );
    }

    return _current;
  }
}
