import 'package:flutter/material.dart';
import 'package:liquid_progress_indicator/liquid_progress_indicator.dart';

import '../../mcg_extension.dart';

class OverProgressWidget extends StatefulWidget {
  OverProgressWidget();

  @override
  OverProgressWidgetState createState() => OverProgressWidgetState();
}

class OverProgressWidgetState extends State<OverProgressWidget> {
  bool _isVisible = false;
  OverController get _overController => Get.find<OverController>();
  @override
  void initState() {
    _overController.overProgressWidgetState = this;
    Future.microtask(() {
      setState(() {
        _isVisible = true;
      });
    });
    super.initState();
  }

  Future<void> close() async {
    if (mounted)
      setState(() {
        _isVisible = false;
      });
    _overController.overProgressWidgetState = null;
    await Future.delayed(OverExtension.kSheetAnimationDuration);
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedContainer(
      duration: OverExtension.kSheetAnimationDuration,
      color: _isVisible ? Colors.black.withAlpha(120) : Colors.black.withAlpha(0),
      child: AnimatedOpacity(
        duration: OverExtension.kSheetAnimationDuration,
        opacity: _isVisible ? 1.0 : 0.0,
        child: Container(
          width: double.infinity,
          height: double.infinity,
          alignment: Alignment.center,
          child: _ProgressWidget(
            text: _overController.progressText,
          ),
        ),
      ),
    );
  }
}

class _ProgressWidget extends StatelessWidget {
  final String? text;

  _ProgressWidget({
    this.text,
  });

  @override
  Widget build(BuildContext context) {
    // final _color = MyPalette.getChartColorFromCount(25.random);
    final _color = Fav.design.primary;
    final _controller = Get.find<OverController>();
    Widget _current = Container(
      alignment: Alignment.center,
      child: Container(
        decoration: ShapeDecoration(color: Colors.white.withAlpha(180), shape: const StadiumBorder()),
        padding: const EdgeInsets.all(8),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            CircularProgressIndicator(valueColor: AlwaysStoppedAnimation(_color)),
            const SizedBox(width: 16),
            SizedBox(
              height: 45,
              width: 200,
              child: Material(
                color: Colors.transparent,
                child: Obx(() => LiquidLinearProgressIndicator(
                    value: _controller.progressValue.value,
                    valueColor: AlwaysStoppedAnimation(_color),
                    backgroundColor: Colors.white,
                    borderColor: _color,
                    borderWidth: 1.0,
                    borderRadius: 22.0,
                    direction: Axis.horizontal, //
                    center: Text((text != null ? text.translate : '') + ' %${((_controller.progressValue.value) * 100).ceil()}', style: const TextStyle(color: Color(0xffdddddd))))),
              ),
            )
          ],
        ),
      ),
    );

    return _current;
  }
}
