import 'package:permission_handler/permission_handler.dart';

import '../mcg_extension.dart';

class PermissionManager {
  PermissionManager._();

  static Future<bool> storage2({bool showAlert = true, FileType fileType = FileType.ANY}) async {
    if (isAndroid) {
      if ((await DeviceManager.androidSDKVersionNo()) < 33) {
        return _checkAndRequestPermission(Permission.storage, showAlert: showAlert);
      } else {
        if (fileType == FileType.IMAGE) {
          return _checkAndRequestPermission(Permission.photos, showAlert: showAlert);
        } else if (fileType == FileType.VIDEO) {
          return _checkAndRequestPermission(Permission.videos, showAlert: showAlert);
        } else {
          return true;
        }
      }
    } else {
      return _checkAndRequestPermission(Permission.storage, showAlert: showAlert);
    }
  }

  static Future<bool> microphone({bool showAlert = true}) async => _checkAndRequestPermission(Permission.microphone, showAlert: showAlert);
  static Future<bool> camera({bool showAlert = true}) async => _checkAndRequestPermission(Permission.camera, showAlert: showAlert);
  static Future<bool> location({bool showAlert = true}) async => _checkAndRequestPermission(Permission.location, showAlert: showAlert);
  static Future<bool> locationWhenInUse({bool showAlert = true}) async => _checkAndRequestPermission(Permission.locationWhenInUse, showAlert: showAlert);
  static Future<bool> photos({bool showAlert = true}) async => _checkAndRequestPermission(Permission.photos, showAlert: showAlert);
  static Future<bool> photosAddOnly({bool showAlert = true}) async => _checkAndRequestPermission(Permission.photosAddOnly, showAlert: showAlert);

  static Future<bool> photoSave({bool showAlert = true}) async {
    if (isAndroid) return storage2(showAlert: showAlert, fileType: FileType.IMAGE);
    //todo bu test edilmesi lazim
    return photosAddOnly(showAlert: showAlert);
  }

  static Future<bool> microphoneAndCamera({bool showAlert = true}) async {
    var _cam = await camera(showAlert: showAlert);
    if (_cam == false) return false;
    return microphone(showAlert: showAlert);
  }

  static Future<bool> _checkAndRequestPermission(Permission permission, {bool showAlert = true}) async {
    log('$permission izni isteniyor');

    var isPermissionAllow = false;

    final status = await permission.status;

    if (status.isGranted) return true;
    final newStatus = await permission.request();
    if (newStatus.isGranted) return true;
    if (showAlert) {
      Over.sure(
        title: 'warning'.translate,
        yesText: 'openpermissionpage'.translate,
        message: 'permissionnotgranted'.translate + ' (${permission.toString()})',
      ).then((value) {
        if (value == true) openAppSettings();
      }).unawaited;
      //  Alert.advance(
      //         title: '',
      //         message: 'permissionnotgranted'.translate + ' (${permission.toString()})',
      //         rowActions: [
      //           AlertAction(
      //             text: 'openpermissionpage'.translate,
      //             onPressed: openAppSettings,
      //           ),
      //         ],
      //         closeButton: true)
      //     .unawaited;
    }

    return isPermissionAllow;
  }

  static Future<bool> hasBackgroundLocationPermission() async {
    final status = await Permission.locationAlways.status;
    return status.isGranted;
  }
}
